@@ -48,7 +48,7 @@ namespace Banshee.GStreamer
         {
         }
         
-        [DllImport ("libbanshee")]
+        [DllImport ("libbanshee.dll")]
         private static extern void gstreamer_initialize (bool debugging, BansheeLogHandler log_handler);
         
         void IExtensionService.Initialize ()
@@ -58,8 +58,34 @@ namespace Banshee.GStreamer
                 native_log_handler = new BansheeLogHandler (NativeLogHandler);
             }
             
-            gstreamer_initialize (debugging, native_log_handler);
+            // Setup the gst plugins/registry paths if running Windows
+            if (!Banshee.Base.PlatformHacks.IsRunningUnix) {
+                string [] gst_paths = new String [] {
+                    "gst-plugins"
+                };
             
+                System.Environment.SetEnvironmentVariable ("GST_PLUGIN_PATH", String.Join (";", gst_paths));
+                System.Environment.SetEnvironmentVariable ("GST_PLUGIN_SYSTEM_PATH", "");
+                System.Environment.SetEnvironmentVariable ("GST_DEBUG", "1");
+
+                string registry = "registry.bin";
+                if (!System.IO.File.Exists (registry)) {
+                    System.IO.File.Create (registry).Close ();
+                }
+
+                System.Environment.SetEnvironmentVariable ("GST_REGISTRY", registry);
+
+                //System.Environment.SetEnvironmentVariable ("GST_REGISTRY_FORK", "no");
+                Console.WriteLine ("GST_PLUGIN_PATH = {0}", System.Environment.GetEnvironmentVariable ("GST_PLUGIN_PATH"));
+            }
+
+            gstreamer_initialize (debugging, native_log_handler);
+
+            ServiceManager.MediaProfileManager.Initialized += OnMediaProfileManagerInitialized;
+        }
+
+        private void OnMediaProfileManagerInitialized (object o, EventArgs args)
+        {
             MediaProfileManager profile_manager = ServiceManager.MediaProfileManager;
             if (profile_manager != null) {
                 Pipeline.AddSExprFunction ("gst-element-is-available", SExprTestElement);
@@ -109,7 +135,7 @@ namespace Banshee.GStreamer
             args.ProfileAvailable = available;
         }
         
-        [DllImport ("libbanshee")]
+        [DllImport ("libbanshee.dll")]
         private static extern bool gstreamer_test_pipeline (IntPtr pipeline);
         
         internal static bool TestPipeline (string pipeline)
