@@ -57,6 +57,7 @@ namespace Banshee.Library
                   <sort-column direction=""asc"">track_title</sort-column>
                   <column modify-default=""ArtistColumn"">
                     <title>{0}</title>
+                    <long-title>{0}</long-title>
                   </column>
                 </column-controller>
             ", Catalog.GetString ("Produced By")));
@@ -69,6 +70,10 @@ namespace Banshee.Library
         protected override bool HasArtistAlbum {
             get { return false; }
         }
+        
+        public override string DefaultBaseDirectory {
+            get { return Banshee.Base.XdgBaseDirectorySpec.GetUserDirectory ("XDG_VIDEOS_DIR", "Videos"); }
+        }
 
         public override IEnumerable<SmartPlaylistDefinition> DefaultSmartPlaylists {
             get { return default_smart_playlists; }
