@@ -47,7 +47,8 @@ namespace Banshee.MediaEngine
 {
     public delegate bool TrackInterceptHandler (TrackInfo track);
 
-    public class PlayerEngineService : IInitializeService, IRequiredService, IPlayerEngineService, IDisposable
+    public class PlayerEngineService : IInitializeService, IDelayedInitializeService, 
+        IRequiredService, IPlayerEngineService, IDisposable
     {   
         private List<PlayerEngine> engines = new List<PlayerEngine> ();
         private PlayerEngine active_engine;
@@ -55,11 +56,14 @@ namespace Banshee.MediaEngine
         private PlayerEngine pending_engine;
         private object pending_playback_for_not_ready;
         private bool pending_playback_for_not_ready_play;
+        private TrackInfo synthesized_contacting_track;
 
         private string preferred_engine_id = null;
 
         public event EventHandler PlayWhenIdleRequest;
         public event TrackInterceptHandler TrackIntercept;
+        public event Action<PlayerEngine> EngineBeforeInitialize;
+        public event Action<PlayerEngine> EngineAfterInitialize;
         
         private event DBusPlayerEventHandler dbus_event_changed;
         event DBusPlayerEventHandler IPlayerEngineService.EventChanged {
@@ -77,7 +81,7 @@ namespace Banshee.MediaEngine
         {
         }
         
-        public void Initialize ()
+        void IInitializeService.Initialize ()
         {
             preferred_engine_id = EngineSchema.Get();
             
@@ -108,7 +112,33 @@ namespace Banshee.MediaEngine
             
             MetadataService.Instance.HaveResult += OnMetadataServiceHaveResult;
             
-            TrackInfo.IsPlayingMethod = IsPlaying;
+            TrackInfo.IsPlayingMethod = track => IsPlaying (track) &&
+                ServiceManager.PlaybackController.Source == ServiceManager.SourceManager.ActiveSource;
+        }
+
+        private void InitializeEngine (PlayerEngine engine)
+        {
+            var handler = EngineBeforeInitialize;
+            if (handler != null) {
+                handler (engine);
+            }
+            
+            engine.Initialize ();
+            engine.IsInitialized = true;
+            
+            handler = EngineAfterInitialize;
+            if (handler != null) {
+                handler (engine);
+            }
+        }
+        
+        void IDelayedInitializeService.DelayedInitialize ()
+        {
+            foreach (var engine in Engines) {
+                if (engine.DelayedInitialize) {
+                    InitializeEngine (engine);
+                }
+            }
         }
         
         private void LoadEngine (TypeExtensionNode node)
@@ -118,6 +148,10 @@ namespace Banshee.MediaEngine
         
         private void LoadEngine (PlayerEngine engine)
         {
+            if (!engine.DelayedInitialize) {
+                InitializeEngine (engine);
+            }
+            
             engine.EventChanged += OnEngineEventChanged;
 
             if (engine.Id == preferred_engine_id) {
@@ -336,12 +370,11 @@ namespace Banshee.MediaEngine
         public void IncrementLastPlayed ()
         {
             if (!incremented_last_played && CurrentTrack != null && CurrentTrack.PlaybackError == StreamPlaybackError.None) {
-                // If we're at least 50% done playing a song, mark it as played, otherwise as skipped
-                // If the Length <= 0, assume the song was finished and increment its play count
-                if (active_engine.Length <= 0 || active_engine.Position >= active_engine.Length / 2) {
-                    active_engine.CurrentTrack.IncrementPlayCount ();
+                //if Length <= 0 assume 100% completion:
+                if (active_engine.Length <= 0) {
+                    CurrentTrack.OnPlaybackFinished (1);
                 } else {
-                    active_engine.CurrentTrack.IncrementSkipCount ();
+                    CurrentTrack.OnPlaybackFinished ((double)active_engine.Position / (double)active_engine.Length);
                 }
                 incremented_last_played = true;
             }
@@ -406,23 +439,44 @@ namespace Banshee.MediaEngine
                 active_engine.Pause ();
             }
         }
+
+        // For use by RadioTrackInfo
+        // TODO remove this method once RadioTrackInfo playlist downloading/parsing logic moved here?
+        internal void StartSynthesizeContacting (TrackInfo track)
+        {
+            //OnStateChanged (PlayerState.Contacting);
+            RaiseEvent (new PlayerEventStateChangeArgs (CurrentState, PlayerState.Contacting));
+            synthesized_contacting_track = track;
+        }
+
+        internal void EndSynthesizeContacting (TrackInfo track, bool idle)
+        {
+            if (track == synthesized_contacting_track) {
+                synthesized_contacting_track = null;
+
+                if (idle) {
+                    RaiseEvent (new PlayerEventStateChangeArgs (PlayerState.Contacting, PlayerState.Idle));
+                }
+            }
+        }
         
         public void TogglePlaying ()
         {
-            if (CurrentState == PlayerState.Playing) {
+            if (IsPlaying () && CurrentState != PlayerState.Paused) {
                 Pause ();
-            } else {
+            } else if (CurrentState != PlayerState.NotReady) {
                 Play ();
             }
         }
         
-        public void VideoExpose (IntPtr window, bool direct)
+        public void VideoExpose (IntPtr displayContext, bool direct)
         {
-            active_engine.VideoExpose (window, direct);
+            active_engine.VideoExpose (displayContext, direct);
         }
         
-        public IntPtr VideoWindow {
-            set { active_engine.VideoWindow = value; }
+        public IntPtr VideoDisplayContext {
+            set { active_engine.VideoDisplayContext = value; }
+            get { return active_engine.VideoDisplayContext; }
         }
         
         public void TrackInfoUpdated ()
@@ -439,7 +493,9 @@ namespace Banshee.MediaEngine
         {
             return CurrentState == PlayerState.Playing || 
                 CurrentState == PlayerState.Paused || 
-                CurrentState == PlayerState.Loaded;
+                CurrentState == PlayerState.Loaded ||
+                CurrentState == PlayerState.Loading ||
+                CurrentState == PlayerState.Contacting;
         }
 
         private void CheckPending ()
@@ -455,7 +511,7 @@ namespace Banshee.MediaEngine
         }
     
         public TrackInfo CurrentTrack {
-            get { return active_engine.CurrentTrack; }
+            get { return active_engine.CurrentTrack ?? synthesized_contacting_track; }
         }
         
         private Dictionary<string, object> dbus_sucks;
@@ -481,7 +537,7 @@ namespace Banshee.MediaEngine
         }
         
         public PlayerState CurrentState {
-            get { return active_engine.CurrentState; }
+            get { return synthesized_contacting_track != null ? PlayerState.Contacting : active_engine.CurrentState; }
         }
         
         string IPlayerEngineService.CurrentState {
@@ -522,8 +578,8 @@ namespace Banshee.MediaEngine
             get { return ((active_engine is IEqualizer) && active_engine.SupportsEqualizer); }
         }
         
-        public bool SupportsVideo {
-            get { return active_engine.SupportsVideo; }
+        public VideoDisplayContextType VideoDisplayContextType {
+            get { return active_engine.VideoDisplayContextType; }
         }
         
         public uint Length {
@@ -531,11 +587,11 @@ namespace Banshee.MediaEngine
                 uint length = active_engine.Length;
                 if (length > 0) {
                     return length;
-                } else if (active_engine.CurrentTrack == null) {
+                } else if (CurrentTrack == null) {
                     return 0;
                 }
                 
-                return (uint) active_engine.CurrentTrack.Duration.TotalSeconds;
+                return (uint) CurrentTrack.Duration.TotalSeconds;
             }
         }
     
