@@ -31,6 +31,7 @@ using System.Collections.Generic;
 using Mono.Unix;
 
 using Hyena;
+using Hyena.Jobs;
 using Hyena.Data;
 using Hyena.Query;
 using Hyena.Data.Sqlite;
@@ -174,6 +175,20 @@ namespace Banshee.Sources
             get { return is_local; }
             protected set { is_local = value; }
         }
+        
+        private static SourceSortType[] sort_types = new SourceSortType[] {
+            SortNameAscending,
+            SortSizeAscending,
+            SortSizeDescending
+        };
+        
+        public override SourceSortType[] ChildSortTypes {
+            get { return sort_types; }
+        }
+        
+        public override SourceSortType DefaultChildSort {
+            get { return SortNameAscending; }
+        }
 
         public delegate void TrackEventHandler (Source sender, TrackEventArgs args);
 
@@ -186,30 +201,20 @@ namespace Banshee.Sources
         {
             return (primary_sources.ContainsKey (id)) ? primary_sources[id] : null;
         }
-        
-        public virtual SafeUri UriAndTypeToSafeUri (TrackUriType type, string uri_field)
-        {
-            if (type == TrackUriType.RelativePath && BaseDirectory != null)
-                return new SafeUri (System.IO.Path.Combine (BaseDirectory, uri_field));
-            else
-                return new SafeUri (uri_field);
-        }
-
-        public virtual void UriToFields (SafeUri uri, out TrackUriType type, out string uri_field)
-        {
-            uri_field = Paths.MakePathRelative (uri.AbsolutePath, BaseDirectory);
-            type = (uri_field == null) ? TrackUriType.AbsoluteUri : TrackUriType.RelativePath;
-            if (uri_field == null) {
-                uri_field = uri.AbsoluteUri;
-            }
-        }
 
         public virtual string BaseDirectory {
             get { return null; }
+            protected set { base_dir_with_sep = null; }
+        }
+        
+        private string base_dir_with_sep;
+        public string BaseDirectoryWithSeparator {
+            get { return base_dir_with_sep ?? (base_dir_with_sep = BaseDirectory + System.IO.Path.DirectorySeparatorChar); }
         }
 
         protected PrimarySource (string generic_name, string name, string id, int order) : base (generic_name, name, id, order)
         {
+            Properties.SetString ("SortChildrenActionLabel", Catalog.GetString ("Sort Playlists by"));
             PrimarySourceInitialize ();
         }
 
@@ -456,6 +461,9 @@ namespace Banshee.Sources
                 Log.WarningFormat ("Cannot delete all tracks from {0} via primary source {1}", source, this);
                 return;
             }
+
+            if (source.Count < 1)
+                return;
             
             ThreadAssist.SpawnFromMain (delegate {
                 CachedList<DatabaseTrackInfo> list = CachedList<DatabaseTrackInfo>.CreateFromModel (source.DatabaseTrackModel);
@@ -465,7 +473,7 @@ namespace Banshee.Sources
 
         protected override void DeleteSelectedTracks (DatabaseTrackListModel model)
         {
-            if (model == null) {
+            if (model == null || model.Count < 1) {
                 return;
             }
             
@@ -577,6 +585,12 @@ namespace Banshee.Sources
             return true;
         }
 
+        private static HyenaSqliteCommand get_track_id_cmd = new HyenaSqliteCommand ("SELECT TrackID FROM CoreTracks WHERE PrimarySourceId = ? AND Uri = ? LIMIT 1");
+        public int GetTrackIdForUri (string uri)
+        {
+            return ServiceManager.DbConnection.Query<int> (get_track_id_cmd, DbId, new SafeUri (uri).AbsoluteUri);
+        }
+
         private bool is_adding;
         public bool IsAdding {
             get { return is_adding; }
@@ -658,10 +672,10 @@ namespace Banshee.Sources
             set { delay_add_job = value; }
         }
 
-        private bool delay_delete_jbo = true;
+        private bool delay_delete_job = true;
         protected bool DelayDeleteJob {
-            get { return delay_delete_jbo; }
-            set { delay_delete_jbo = value; }
+            get { return delay_delete_job; }
+            set { delay_delete_job = value; }
         }
 
         private BatchUserJob add_track_job;
@@ -672,6 +686,8 @@ namespace Banshee.Sources
                         add_track_job = new BatchUserJob (String.Format (Catalog.GetString (
                             "Adding {0} of {1} to {2}"), "{0}", "{1}", Name), 
                             Properties.GetStringList ("Icon.Name"));
+                        add_track_job.SetResources (Resource.Cpu, Resource.Database, Resource.Disk);
+                        add_track_job.PriorityHints = PriorityHints.SpeedSensitive | PriorityHints.DataLossIfStopped;
                         add_track_job.DelayShow = DelayAddJob;
                         add_track_job.CanCancel = true;
                         add_track_job.Register ();
@@ -689,6 +705,8 @@ namespace Banshee.Sources
                         delete_track_job = new BatchUserJob (String.Format (Catalog.GetString (
                             "Deleting {0} of {1} From {2}"), "{0}", "{1}", Name),
                             Properties.GetStringList ("Icon.Name"));
+                        delete_track_job.SetResources (Resource.Cpu, Resource.Database);
+                        delete_track_job.PriorityHints = PriorityHints.SpeedSensitive | PriorityHints.DataLossIfStopped;
                         delete_track_job.DelayShow = DelayDeleteJob;
                         delete_track_job.Register ();
                     }
