@@ -54,7 +54,7 @@ namespace Banshee.Gui
                     Catalog.GetString ("Import media from a variety of sources"), OnImport),
 
                 new ActionEntry ("ImportPlaylistAction", null,
-                    Catalog.GetString ("Import Playlist..."), null,
+                    Catalog.GetString ("Import _Playlist..."), null,
                     Catalog.GetString ("Import a playlist"), OnImportPlaylist),
 
                 new ActionEntry ("RescanAction", null,
@@ -168,13 +168,14 @@ namespace Banshee.Gui
             chooser.DefaultResponse = ResponseType.Ok;
             chooser.SelectMultiple = true;
             chooser.AddFilter (Hyena.Gui.GtkUtilities.GetFileFilter (Catalog.GetString ("Playlists"), PlaylistFileUtil.PlaylistExtensions));
-            try {
+            // TODO library-location add the BaseDirectory for the library being imported to (or all of them)
+            /*try {
                 chooser.AddShortcutFolder (Paths.LibraryLocation);
-            } catch {}
+            } catch {}*/
 
             chooser.AddButton (Stock.Cancel, ResponseType.Cancel);
             // Translators: verb
-            chooser.AddButton (Catalog.GetString("Import"), ResponseType.Ok);
+            chooser.AddButton (Catalog.GetString("I_mport"), ResponseType.Ok);
             
             int response = chooser.Run();            
 
