@@ -143,8 +143,10 @@ namespace Banshee.Gui
 
             switch (args.Current) {
                 case PlayerState.Contacting:
+                case PlayerState.Loading:
+                case PlayerState.Loaded:
                 case PlayerState.Playing:
-                    ShowPlayAction ();
+                    ShowStopAction ();
                     break;
                 case PlayerState.Paused:
                     ShowPlay ();
@@ -161,18 +163,21 @@ namespace Banshee.Gui
                 this["SeekToAction"].Sensitive = !track.IsLive;
                 this["RestartSongAction"].Sensitive = !track.IsLive;
 
-                this["RestartSongAction"].Label = String.Format (Catalog.GetString ("_Restart {0}"), track.MediaTypeName);
-                this["JumpToPlayingTrackAction"].Label = String.Format (Catalog.GetString ("_Jump to Playing {0}"), track.MediaTypeName);
+                this["RestartSongAction"].Label = track.RestartLabel;
+                this["JumpToPlayingTrackAction"].Label = track.JumpToLabel;
                 this["JumpToPlayingTrackAction"].Sensitive = true;
             } else {
                 this["JumpToPlayingTrackAction"].Sensitive = false;
                 this["RestartSongAction"].Sensitive = false;
                 this["SeekToAction"].Sensitive = false;
             }
+
+            // Disable all actions while NotReady
+            Sensitive = args.Current != PlayerState.NotReady;
         }
         
-        private void ShowPlayAction ()
-        { 
+        private void ShowStopAction ()
+        {
             if (ServiceManager.PlayerEngine.CanPause) {
                 ShowPause ();
             } else {
@@ -210,7 +215,12 @@ namespace Banshee.Gui
         
         private void OnPreviousAction (object o, EventArgs args)
         {
-            ServiceManager.PlaybackController.Previous ();
+            const int delay = 4000; // ms
+            if (ServiceManager.PlayerEngine.Position < delay) {
+                ServiceManager.PlaybackController.Previous ();
+            } else {
+                OnRestartSongAction (o, args);
+            }
         }
 
         private void OnSeekToAction (object o, EventArgs args)
@@ -251,6 +261,7 @@ namespace Banshee.Gui
                         track_src.TrackModel.Selection.Select (i);
                         track_src.TrackModel.Selection.FocusedIndex = i;
                         track_list.CenterOn (i);
+                        track_list.GrabFocus ();
                     }
                 }
             }
