@@ -58,7 +58,9 @@ namespace Banshee.NotificationArea
         private Menu menu;
         private RatingMenuItem rating_menu_item;
         private BansheeActionGroup actions;
-        private uint ui_manager_id;
+        private bool? actions_supported;
+        
+        private int ui_manager_id;
         
         private bool show_notifications;
         private string notify_last_title;
@@ -106,6 +108,13 @@ namespace Banshee.NotificationArea
                 return false;
             }
             
+            GLib.Timeout.Add (1000, delegate {
+                if (notif_area != null) {
+                    notif_area.Show ();
+                }
+                return false;
+            });
+            
             return true;
         }
         
@@ -126,37 +135,8 @@ namespace Banshee.NotificationArea
             
             interface_action_service.AddActionGroup (actions);
             
-            ui_manager_id = interface_action_service.UIManager.AddUiFromResource ("NotificationAreaMenu.xml");      
-            menu = (Menu)interface_action_service.UIManager.GetWidget("/NotificationAreaIconMenu");
-            menu.Show ();
+            ui_manager_id = -1;
             
-            for (int i = 0; i < menu.Children.Length; i++) {
-                if (menu.Children[i].Name == "Previous") {
-                    int j = i;
-                    PlaybackRepeatActions repeat_group = interface_action_service.FindActionGroup ("PlaybackRepeat")
-                         as PlaybackRepeatActions;
-                    if (repeat_group != null) {
-                        menu.Insert (repeat_group.CreateSubmenu (), i++ + 2);
-                    }
-                    
-                    PlaybackShuffleActions shuffle_group = interface_action_service.FindActionGroup ("PlaybackShuffle")
-                         as PlaybackShuffleActions;
-                    if (shuffle_group != null) {
-                        menu.Insert (shuffle_group.CreateSubmenu (), i++ + 2);
-                    }
-                    
-                    if (j != i) {
-                        menu.Insert (new SeparatorMenuItem (), i++ + 2);
-                    }
-                    
-                    rating_menu_item = new RatingMenuItem ();
-                    rating_menu_item.Activated += OnRatingChanged;
-                    ToggleRatingMenuSensitive ();
-                    menu.Insert (rating_menu_item, i + 2);
-                    break;
-                }
-            }
-
             ServiceManager.PlayerEngine.ConnectEvent (OnPlayerEvent,
                PlayerEvent.StartOfStream |
                PlayerEvent.EndOfStream |
@@ -191,8 +171,11 @@ namespace Banshee.NotificationArea
                 interface_action_service.GlobalActions.Remove (close_action);
             }
             
-            interface_action_service.RemoveActionGroup ("NotificationArea");
-            interface_action_service.UIManager.RemoveUi (ui_manager_id);
+            if (ui_manager_id >= 0) {
+                interface_action_service.RemoveActionGroup ("NotificationArea");
+                interface_action_service.UIManager.RemoveUi ((uint)ui_manager_id);
+                ui_manager_id = -1;
+            }
             
             actions = null;
             elements_service = null;
@@ -200,6 +183,17 @@ namespace Banshee.NotificationArea
             
             disposed = true;
         }
+
+        private bool ActionsSupported {
+            get {
+                if (!actions_supported.HasValue) {
+                    actions_supported = Notifications.Global.Capabilities != null &&
+                        Array.IndexOf (Notifications.Global.Capabilities, "actions") > -1;
+                }
+
+                return actions_supported.Value;
+            }
+        }
         
         private bool BuildNotificationArea () 
         {
@@ -240,6 +234,44 @@ namespace Banshee.NotificationArea
             }
         }
         
+        private void BuildContextMenu ()
+        {
+            if (menu != null) {
+                return;
+            }
+            
+            ui_manager_id = (int)interface_action_service.UIManager.AddUiFromResource ("NotificationAreaMenu.xml");
+            menu = (Menu)interface_action_service.UIManager.GetWidget("/NotificationAreaIconMenu");
+            menu.Show ();
+            
+            for (int i = 0; i < menu.Children.Length; i++) {
+                if (menu.Children[i].Name == "Previous") {
+                    int j = i;
+                    PlaybackRepeatActions repeat_group = interface_action_service.FindActionGroup ("PlaybackRepeat")
+                         as PlaybackRepeatActions;
+                    if (repeat_group != null) {
+                        menu.Insert (repeat_group.CreateSubmenu (), i++ + 2);
+                    }
+                    
+                    PlaybackShuffleActions shuffle_group = interface_action_service.FindActionGroup ("PlaybackShuffle")
+                         as PlaybackShuffleActions;
+                    if (shuffle_group != null) {
+                        menu.Insert (shuffle_group.CreateSubmenu (), i++ + 2);
+                    }
+                    
+                    if (j != i) {
+                        menu.Insert (new SeparatorMenuItem (), i++ + 2);
+                    }
+                    
+                    rating_menu_item = new RatingMenuItem ();
+                    rating_menu_item.Activated += OnRatingChanged;
+                    ToggleRatingMenuSensitive ();
+                    menu.Insert (rating_menu_item, i + 2);
+                    break;
+                }
+            }
+        }
+        
         private void RegisterCloseHandler ()
         {
             if (elements_service.PrimaryWindowClose == null) {
@@ -274,6 +306,8 @@ namespace Banshee.NotificationArea
         
         private void OnNotificationAreaPopupMenuEvent (object o, PopupMenuArgs args)
         {
+            BuildContextMenu ();
+
             if (rating_menu_item.Visible) {
                 TrackInfo track = ServiceManager.PlayerEngine.CurrentTrack;
                 if (track != null) {
@@ -283,6 +317,7 @@ namespace Banshee.NotificationArea
                     rating_menu_item.Reset (track.Rating);
                 }
             }
+
             menu.Popup (null, null, notif_area.PositionMenu, 3, Gtk.Global.CurrentEventTime);
         }
         
@@ -353,12 +388,23 @@ namespace Banshee.NotificationArea
         
         private void ToggleRatingMenuSensitive () 
         {
+            if (rating_menu_item == null) {
+                return;
+            }
+            
             if (ServiceManager.PlayerEngine.CurrentTrack is DatabaseTrackInfo) {
                 rating_menu_item.Show ();
             } else {
                 rating_menu_item.Hide ();
             }
         }
+
+        private bool IsNotificationDaemon {
+            get {
+                var name = Notifications.Global.ServerInformation.Name;
+                return name == "notification-daemon" || name == "Notification Daemon";
+            }
+        }
         
         private void ShowTrackNotification ()
         {
@@ -386,7 +432,11 @@ namespace Banshee.NotificationArea
             Gdk.Pixbuf image = null;
             
             if (artwork_manager_service != null) {
-                image = artwork_manager_service.LookupScalePixbuf (current_track.ArtworkId, 42);
+                if (IsNotificationDaemon) {
+                    image = artwork_manager_service.LookupScalePixbuf (current_track.ArtworkId, 42);
+                } else {
+                    image = artwork_manager_service.LookupPixbuf (current_track.ArtworkId);
+                }
             }
             
             if (image == null) {
@@ -405,7 +455,7 @@ namespace Banshee.NotificationArea
                     message, image, notif_area.Widget);
                 nf.Urgency = Urgency.Low;
                 nf.Timeout = 4500;
-                if (interface_action_service.PlaybackActions["NextAction"].Sensitive) {
+                if (!current_track.IsLive && ActionsSupported && interface_action_service.PlaybackActions["NextAction"].Sensitive) {
                     nf.AddAction ("skip-song", Catalog.GetString("Skip this item"), OnSongSkipped);
                 }
                 nf.Show ();
