@@ -32,11 +32,7 @@ using System.IO;
 using System.Data;
 using System.Text;
 using System.Threading;
-
-// NOTE: Mono.Data.Sqlite has serious threading issues.  You cannot access
-//       its results from any thread but the one the SqliteConnection belongs to.
-//       That is why we still use Mono.Data.SqliteClient.
-using Mono.Data.SqliteClient;
+using Mono.Data.Sqlite;
 
 namespace Hyena.Data.Sqlite
 {
@@ -58,10 +54,11 @@ namespace Hyena.Data.Sqlite
     
     public class HyenaSqliteCommand
     {
-        protected object result = null;
+        private object result = null;
         private Exception execution_exception = null;
         private bool finished = false;
 
+        private ManualResetEvent finished_event = new ManualResetEvent (true);
         private string command;
         private string command_format = null;
         private string command_formatted = null;
@@ -123,7 +120,9 @@ namespace Hyena.Data.Sqlite
 
                 switch (command_type) {
                     case HyenaCommandType.Reader:
-                        result = sql_command.ExecuteReader ();
+                        using (SqliteDataReader reader = sql_command.ExecuteReader ()) {
+                            result = new HyenaSqliteArrayDataReader (reader);
+                        }
                         break;
 
                     case HyenaCommandType.Scalar:
@@ -149,6 +148,7 @@ namespace Hyena.Data.Sqlite
                 execution_exception = e;
             }
 
+            finished_event.Reset ();
             finished = true;
         }
 
@@ -164,6 +164,7 @@ namespace Hyena.Data.Sqlite
             finished = false;
 
             conn.ClaimResult ();
+            finished_event.Set ();
 
             if (execution_exception != null) {
                 throw execution_exception;
@@ -172,6 +173,11 @@ namespace Hyena.Data.Sqlite
             return ret;
         }
 
+        internal void WaitIfNotFinished ()
+        {
+            finished_event.WaitOne ();
+        }
+
         internal HyenaSqliteCommand ApplyValues (params object [] param_values)
         {
             if (command_format == null) {
@@ -201,7 +207,7 @@ namespace Hyena.Data.Sqlite
             return this;
         }
 
-        protected static object SqlifyObject (object o)
+        public static object SqlifyObject (object o)
         {
             if (o is string) {
                 return String.Format ("'{0}'", (o as string).Replace ("'", "''"));
@@ -211,6 +217,9 @@ namespace Hyena.Data.Sqlite
                 return ((bool)o) ? "1" : "0";
             } else if (o == null) {
                 return "NULL";
+            } else if (o is byte[]) {
+                string hex = BitConverter.ToString (o as byte[]).Replace ("-", "");
+                return String.Format ("X'{0}'", hex);
             } else if (o is Array) {
                 StringBuilder sb = new StringBuilder ();
                 bool first = true;
