@@ -43,6 +43,7 @@ namespace Hyena.Data.Sqlite
         private DatabaseColumn key;
         private int key_select_column_index;
         private HyenaSqliteConnection connection;
+        private bool check_table = true;
         
         private HyenaSqliteCommand create_command;
         private HyenaSqliteCommand insert_command;
@@ -86,15 +87,20 @@ namespace Hyena.Data.Sqlite
         public HyenaSqliteConnection Connection {
             get { return connection; }
         }
-        
+
         protected SqliteModelProvider (HyenaSqliteConnection connection)
         {
             this.connection = connection;
         }
+
+        public SqliteModelProvider (HyenaSqliteConnection connection, string table_name) : this (connection, table_name, true)
+        {
+        }
         
-        public SqliteModelProvider (HyenaSqliteConnection connection, string table_name) : this (connection)
+        public SqliteModelProvider (HyenaSqliteConnection connection, string table_name, bool checkTable) : this (connection)
         {
             this.table_name = table_name;
+            this.check_table = checkTable;
             Init ();
         }
 
@@ -126,8 +132,10 @@ namespace Hyena.Data.Sqlite
             
             key_select_column_index = select_columns.IndexOf (key);
             
-            CheckVersion ();
-            CheckTable ();
+            if (check_table) {
+                CheckVersion ();
+                CheckTable ();
+            }
         }
         
         protected virtual void CheckVersion ()
@@ -213,36 +221,8 @@ namespace Hyena.Data.Sqlite
                 DatabaseColumn c = member is FieldInfo
                     ? new DatabaseColumn ((FieldInfo)member, column)
                     : new DatabaseColumn ((PropertyInfo)member, column);
-                
-                foreach (DatabaseColumn col in columns) {
-                    if (col.Name == c.Name) {
-                        throw new Exception (String.Format (
-                            "{0} has multiple columns named {1}",
-                             TableName, c.Name)
-                        );
-                    }
-                    if (col.Index != null && col.Index == c.Index) {
-                        throw new Exception (String.Format (
-                            "{0} has multiple indecies named {1}",
-                            TableName, c.Name)
-                        );
-                    }
-                }
-                
-                columns.Add (c);
-
-                if (column.Select) {
-                    select_columns.Add (c);
-                }
-                
-                if ((c.Constraints & DatabaseColumnConstraints.PrimaryKey) > 0) {
-                    if (key != null) {
-                        throw new Exception (String.Format (
-                            "Multiple primary keys in the {0} table", TableName)
-                        );
-                    }
-                    key = c;
-                }
+                    
+                AddColumn (c, column.Select);
             }
             VirtualDatabaseColumnAttribute virtual_column = attribute as VirtualDatabaseColumnAttribute;
             if (virtual_column != null) {
@@ -254,6 +234,39 @@ namespace Hyena.Data.Sqlite
             }
         }
         
+        protected void AddColumn (DatabaseColumn c, bool select)
+        {
+            foreach (DatabaseColumn col in columns) {
+                if (col.Name == c.Name) {
+                    throw new Exception (String.Format (
+                        "{0} has multiple columns named {1}",
+                         TableName, c.Name)
+                    );
+                }
+                if (col.Index != null && col.Index == c.Index) {
+                    throw new Exception (String.Format (
+                        "{0} has multiple indecies named {1}",
+                        TableName, c.Name)
+                    );
+                }
+            }
+            
+            columns.Add (c);
+
+            if (select) {
+                select_columns.Add (c);
+            }
+            
+            if ((c.Constraints & DatabaseColumnConstraints.PrimaryKey) > 0) {
+                if (key != null) {
+                    throw new Exception (String.Format (
+                        "Multiple primary keys in the {0} table", TableName)
+                    );
+                }
+                key = c;
+            }
+        }
+        
         protected virtual void CreateTable ()
         {
             connection.Execute (CreateCommand);
@@ -378,13 +391,18 @@ namespace Hyena.Data.Sqlite
         
         public IEnumerable<T> FetchAllMatching (string condition, params object [] vals)
         {
-            HyenaSqliteCommand fetch_matching_command = new HyenaSqliteCommand (String.Format ("{0} AND {1}", SelectCommand.Text, condition));
+            HyenaSqliteCommand fetch_matching_command = CreateFetchCommand (condition);
             using (IDataReader reader = connection.Query (fetch_matching_command, vals)) {
                 while (reader.Read ()) {
                     yield return Load (reader);
                 }
             }
         }
+
+        public HyenaSqliteCommand CreateFetchCommand (string condition)
+        {
+            return new HyenaSqliteCommand (String.Format ("{0} AND {1}", SelectCommand.Text, condition));
+        }
         
         public IEnumerable<T> FetchRange (int offset, int limit)
         {
@@ -569,7 +587,7 @@ namespace Hyena.Data.Sqlite
         protected virtual HyenaSqliteCommand SelectRangeCommand {
             get {
                 if (select_range_command == null) {
-                    select_command = new HyenaSqliteCommand (
+                    select_range_command = new HyenaSqliteCommand (
                         String.Format (
                             "SELECT {0} FROM {1}{2}{3} LIMIT ?, ?",
                             Select, From,
@@ -643,6 +661,7 @@ namespace Hyena.Data.Sqlite
                 }
                 return primary_key;
             }
+            protected set { primary_key = value; }
         }
         
         private void BuildQuerySql ()
@@ -662,44 +681,42 @@ namespace Hyena.Data.Sqlite
             
             StringBuilder where_builder = new StringBuilder ();
             Dictionary<string, string> tables = new Dictionary<string,string> (virtual_columns.Count + 1);
-            tables.Add (TableName, null);
             bool first_virtual = true;
             foreach (VirtualDatabaseColumn column in virtual_columns) {
-                if (first_virtual) {
-                    first_virtual = false;
-                } else {
-                    where_builder.Append (" AND ");
-                }
                 if (first) {
                     first = false;
                 } else {
                     select_builder.Append (',');
                 }
+
                 select_builder.Append (column.TargetTable);
                 select_builder.Append ('.');
                 select_builder.Append (column.Name);
+
+                bool table_not_joined = !tables.ContainsKey (column.TargetTable);
+                if (first_virtual) {
+                    first_virtual = false;
+                } else if (table_not_joined) {
+                    where_builder.Append (" AND ");
+                }
                 
-                where_builder.Append (column.TargetTable);
-                where_builder.Append ('.');
-                where_builder.Append (column.ForeignKey);
-                where_builder.Append (" = ");
-                where_builder.Append (TableName);
-                where_builder.Append ('.');
-                where_builder.Append (column.LocalKey);
+                if (table_not_joined) {
+                    where_builder.Append (column.TargetTable);
+                    where_builder.Append ('.');
+                    where_builder.Append (column.ForeignKey);
+                    where_builder.Append (" = ");
+                    where_builder.Append (TableName);
+                    where_builder.Append ('.');
+                    where_builder.Append (column.LocalKey);
                 
-                if (!tables.ContainsKey (column.TargetTable)) {
                     tables.Add (column.TargetTable, null);
                 }
             }
             
             StringBuilder from_builder = new StringBuilder ();
-            bool first_tables = true;
+            from_builder.Append (TableName);
             foreach (KeyValuePair<string, string> pair in tables) {
-                if (first_tables) {
-                    first_tables = false;
-                } else {
-                    from_builder.Append (',');
-                }
+                from_builder.Append (',');
                 from_builder.Append (pair.Key);
             }
 
